import React, { useReducer, useState, useEffect } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var styles = {"row_cell":"_3TtXw","row":"_32f2G"};

function makeId(length) {
  if (length === void 0) {
    length = 15;
  }

  var text = "";
  var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

  for (var i = 0; i < length; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }

  return text;
}
function evaluatePosition(firstPosition, secondPosition) {
  return {
    row: firstPosition.row,
    column: firstPosition.column,
    length: secondPosition.column - firstPosition.column + 1
  };
}
function getPosition(titleWidth, row, column, dimension) {
  var position = {
    left: titleWidth + column * dimension,
    top: (row + 1) * dimension
  };
  return position;
}
function getColumnCount(dimension, width, rowTitleWidth) {
  return Math.floor((width - rowTitleWidth) / dimension);
}
function getRowCount(dimension, height) {
  return Math.floor(height / dimension);
}
function resizeBar(editBar, bars, newLocation) {
  var editing = bars.filter(function (bar) {
    return bar.editing;
  });
  editing.forEach(function (bar) {
    var nPosition = evaluatePosition({
      column: bar.column,
      row: bar.row
    }, newLocation.cell);
    editBar(_extends({}, bar, nPosition));
  });
}

var actionTypes = {
  add: 'ADD',
  edit: 'EDIT',
  "delete": 'DELETE',
  addToEditing: 'ADD_TO_EDITING',
  doneEditing: 'DONE_EDITING'
};

function reserverReducer(state, action) {
  switch (action.type) {
    case actionTypes.edit:
      {
        var nBars = [].concat(state.bars);
        var bIndex = nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });
        nBars[bIndex] = action.payload;
        return _extends({}, state, {
          bars: nBars
        });
      }

    case actionTypes.add:
      {
        var bars = [].concat(state.bars);
        bars.push(action.payload);
        return _extends({}, state, {
          bars: bars,
          isEditing: true
        });
      }

    case actionTypes["delete"]:
      {
        var _nBars = [].concat(state.bars);

        var _bIndex = _nBars.findIndex(function (bar) {
          return action.payload.id === bar.id;
        });

        _nBars.splice(_bIndex, 1);

        return _extends({}, state, {
          bars: _nBars
        });
      }

    case actionTypes.doneEditing:
      {
        if (state.isEditing) {
          var _nBars2 = [].concat(state.bars);

          _nBars2 = _nBars2.map(function (bar) {
            if (bar.editing) {
              return _extends({}, bar, {
                editing: false,
                pointerEvents: "auto"
              });
            }

            return bar;
          });
          return _extends({}, state, {
            bars: _nBars2,
            isEditing: false
          });
        }

        return state;
      }

    case actionTypes.mouseDragOver:
      {
        return _extends({}, state);
      }

    default:
      {
        console.log("Unhandled type: " + action.type);
        return state;
      }
  }
}

function useReserver(reducer, initialState) {
  var _useReducer = useReducer(reducer, {
    bars: initialState,
    isEditing: false
  }),
      _useReducer$ = _useReducer[0],
      bars = _useReducer$.bars,
      isEditing = _useReducer$.isEditing,
      dispatch = _useReducer[1];

  var addBar = function addBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.add
    });
  };

  var editBar = function editBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes.edit
    });
  };

  var deleteBar = function deleteBar(props) {
    return dispatch({
      payload: props,
      type: actionTypes["delete"]
    });
  };

  var doneEditing = function doneEditing(props) {
    return dispatch({
      type: actionTypes.doneEditing
    });
  };

  var addToEditing = function addToEditing(props) {
    return dispatch({
      payload: props,
      type: actionTypes.addToEditing
    });
  };

  return {
    bars: bars,
    isEditing: isEditing,
    addBar: addBar,
    editBar: editBar,
    doneEditing: doneEditing,
    deleteBar: deleteBar,
    addToEditing: addToEditing
  };
}

function Bar(props) {
  return /*#__PURE__*/React.createElement("div", {
    onDragStart: props.onDragStart,
    onClick: function onClick() {
      props.onClick(props);
    },
    onMouseOver: function onMouseOver() {
      props.onMouseOver(props);
    },
    draggable: props.draggable,
    style: _extends({}, props.style, {
      top: props.top,
      left: props.left,
      display: "flex",
      position: "absolute",
      zIndex: props.zIndex || "100",
      pointerEvents: props.pointerEvents
    }),
    className: props.className
  }, [].concat(Array(props.length && props.length > 0 ? props.length : 1)).map(function (notUsed, i) {
    return /*#__PURE__*/React.createElement("div", {
      key: i,
      style: {
        width: props.dimension,
        height: props.dimension,
        pointerEvents: props.pointerEvents
      }
    });
  }), props.children);
}
Bar.defaultProps = {
  dimension: 20,
  onClick: function onClick() {},
  onMouseOver: function onMouseOver() {},
  onDragStart: function onDragStart() {}
};

function Head(props) {
  var _useState = useState([]),
      headRow = _useState[0],
      setHeadRow = _useState[1];

  useEffect(function () {
    if (typeof props.headRow === "function") {
      setHeadRow(props.headRow(props.columnCount));
    } else if (Array.isArray(props.headRow)) {
      setHeadRow(props.headRow);
    }
  }, [props.headRow, props.columnCount]);
  return /*#__PURE__*/React.createElement("div", {
    style: {
      height: props.dimension,
      display: "flex"
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: styles.row_cell,
    style: {
      width: props.rowTitleWidth,
      height: props.dimension + "px"
    }
  }), headRow.map(function (headitem, i) {
    return /*#__PURE__*/React.createElement("div", {
      key: i,
      style: {
        width: props.dimension + "px",
        height: props.dimension + "px"
      },
      className: styles.row_cell
    }, headitem);
  }));
}

function Reserver(props) {
  var _useState = useState([]),
      rowTitles = _useState[0],
      setRowTitles = _useState[1];

  var _useState2 = useState(0),
      rowCount = _useState2[0],
      setRowCount = _useState2[1];

  var _useState3 = useState(0),
      columnCount = _useState3[0],
      setColumnCount = _useState3[1];

  useEffect(function () {
    setColumnCount(getColumnCount(props.dimension, props.width, props.rowTitleWidth));
  }, [props.width, props.dimension]);
  useEffect(function () {
    setRowCount(getRowCount(props.dimension, props.height));
  }, [props.height, props.dimension]);
  useEffect(function () {
    if (typeof props.rowTitles === "function") {
      setRowTitles(props.rowTitles());
    } else if (Array.isArray(props.rowTitles)) {
      setRowTitles(props.rowTitles);
    }
  }, [props.rowTitles]);

  return /*#__PURE__*/React.createElement("div", {
    onMouseLeave: props.mouseLeaveGrid,
    id: "grid",
    style: _extends({}, props.style, {
      position: "relative"
    })
  }, /*#__PURE__*/React.createElement(Head, {
    headRow: props.headRow,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: props.dimension
  }), [].concat(Array(rowCount)).map(function (x, r) {
    return /*#__PURE__*/React.createElement("div", {
      className: styles.row,
      key: r,
      style: {
        height: props.dimension,
        display: "flex"
      }
    }, /*#__PURE__*/React.createElement("div", {
      className: styles.row_cell,
      style: {
        width: props.rowTitleWidth,
        height: props.dimension + "px"
      }
    }, rowTitles[r]), [].concat(Array(columnCount)).map(function (x, c) {
      return /*#__PURE__*/React.createElement("div", {
        onDragOver: function onDragOver(e) {
          e.preventDefault();
          props.mouseDragOverCell({
            cell: {
              row: r,
              column: c
            }
          });
        },
        className: styles.row_cell,
        id: "carribean_r" + r + "c" + c,
        onMouseEnter: function onMouseEnter() {
          props.mouseEnterCell({
            dimension: props.dimension,
            cell: {
              row: r,
              column: c
            }
          });
        },
        onMouseDown: function onMouseDown() {
          props.mouseDownCell({
            dimension: props.dimension,
            cell: {
              row: r,
              column: c
            }
          });
        },
        onMouseUp: function onMouseUp() {
          console.log("mouseup");
          props.mouseUpCell({
            cell: {
              row: r,
              column: c
            }
          });
        },
        onDrop: function onDrop() {
          props.mouseCellDrop({
            cell: {
              row: r,
              column: c
            }
          });
        },
        style: {
          width: props.dimension + "px",
          height: props.dimension + "px"
        },
        key: c
      }, props.content["r" + r + "c" + c]);
    }));
  }), typeof props.children === "function" && props.children({
    rowCount: rowCount,
    columnCount: columnCount,
    rowTitleWidth: props.rowTitleWidth,
    dimension: props.dimension
  }));
}

function createBar(dimension, startLocation) {
  return _extends({
    id: makeId(),
    dimension: dimension,
    style: {
      background: "orange"
    },
    editing: true
  }, startLocation, {
    length: 1,
    pointerEvents: "none"
  });
}
Reserver.defaultProps = {
  initBars: [],
  bars: [],
  tags: [],
  headRow: [],
  rowTitles: [],
  content: {},
  dimension: 20,
  width: 500,
  height: 500,
  rowTitleWidth: 0,
  mouseEnterCell: function mouseEnterCell() {},
  mouseDownCell: function mouseDownCell() {},
  mouseUpCell: function mouseUpCell() {},
  mouseDragOverCell: function mouseDragOverCell() {},
  mouseCellDrop: function mouseCellDrop() {},
  mouseLeaveGrid: function mouseLeaveGrid() {}
};

export default Reserver;
export { Bar, actionTypes, createBar, evaluatePosition, getPosition, makeId, reserverReducer, resizeBar, useReserver };
//# sourceMappingURL=index.modern.js.map
